@@ -20,7 +20,6 @@
 # USA
 
 import sys
-from gi import _gobject
 from gi.repository import GObject
 from ..overrides import override
 from ..importer import modules
@@ -61,6 +60,7 @@ class Widget(Gtk.Widget):
 Widget = override(Widget)
 __all__.append('Widget')
 
+
 class Container(Gtk.Container, Widget):
 
     def __len__(self):
@@ -86,6 +86,7 @@ class Container(Gtk.Container, Widget):
 Container = override(Container)
 __all__.append('Container')
 
+
 class Editable(Gtk.Editable):
 
     def insert_text(self, text, position):
@@ -103,6 +104,7 @@ class Editable(Gtk.Editable):
 Editable = override(Editable)
 __all__.append("Editable")
 
+
 class Action(Gtk.Action):
     def __init__(self, name, label, tooltip, stock_id, **kwds):
         Gtk.Action.__init__(self, name=name, label=label, tooltip=tooltip, stock_id=stock_id, **kwds)
@@ -110,6 +112,7 @@ class Action(Gtk.Action):
 Action = override(Action)
 __all__.append("Action")
 
+
 class RadioAction(Gtk.RadioAction):
     def __init__(self, name, label, tooltip, stock_id, value, **kwds):
         Gtk.RadioAction.__init__(self, name=name, label=label, tooltip=tooltip, stock_id=stock_id, value=value, **kwds)
@@ -117,9 +120,10 @@ class RadioAction(Gtk.RadioAction):
 RadioAction = override(RadioAction)
 __all__.append("RadioAction")
 
+
 class ActionGroup(Gtk.ActionGroup):
     def __init__(self, name, **kwds):
-        super(ActionGroup, self).__init__(name = name, **kwds)
+        super(ActionGroup, self).__init__(name=name, **kwds)
 
     def add_actions(self, entries, user_data=None):
         """
@@ -215,7 +219,6 @@ class ActionGroup(Gtk.ActionGroup):
             # using inner function above since entries can leave out optional arguments
             _process_action(*e)
 
-
     def add_radio_actions(self, entries, value=None, on_change=None, user_data=None):
         """
         The add_radio_actions() method is a convenience method that creates a
@@ -283,6 +286,7 @@ class ActionGroup(Gtk.ActionGroup):
 ActionGroup = override(ActionGroup)
 __all__.append('ActionGroup')
 
+
 class UIManager(Gtk.UIManager):
     def add_ui_from_string(self, buffer):
         if not isinstance(buffer, _basestring):
@@ -298,6 +302,7 @@ class UIManager(Gtk.UIManager):
 UIManager = override(UIManager)
 __all__.append('UIManager')
 
+
 class ComboBox(Gtk.ComboBox, Container):
 
     def get_active_iter(self):
@@ -318,6 +323,7 @@ class Box(Gtk.Box):
 Box = override(Box)
 __all__.append('Box')
 
+
 class SizeGroup(Gtk.SizeGroup):
     def __init__(self, mode=Gtk.SizeGroupMode.VERTICAL):
         super(SizeGroup, self).__init__(mode=mode)
@@ -386,6 +392,8 @@ __all__.append('Builder')
 
 # NOTE: This must come before any other Window/Dialog subclassing, to ensure
 # that we have a correct inheritance hierarchy.
+
+
 class Window(Gtk.Window):
     def __init__(self, type=Gtk.WindowType.TOPLEVEL, **kwds):
         Gtk.Window.__init__(self, type=type, **kwds)
@@ -459,6 +467,7 @@ class Dialog(Gtk.Dialog, Container):
 Dialog = override(Dialog)
 __all__.append('Dialog')
 
+
 class MessageDialog(Gtk.MessageDialog, Dialog):
     def __init__(self,
                  parent=None,
@@ -495,6 +504,7 @@ class MessageDialog(Gtk.MessageDialog, Dialog):
 MessageDialog = override(MessageDialog)
 __all__.append('MessageDialog')
 
+
 class AboutDialog(Gtk.AboutDialog):
     def __init__(self, **kwds):
         Gtk.AboutDialog.__init__(self, **kwds)
@@ -502,6 +512,7 @@ class AboutDialog(Gtk.AboutDialog):
 AboutDialog = override(AboutDialog)
 __all__.append('AboutDialog')
 
+
 class ColorSelectionDialog(Gtk.ColorSelectionDialog):
     def __init__(self, title=None, **kwds):
         Gtk.ColorSelectionDialog.__init__(self, title=title, **kwds)
@@ -509,6 +520,7 @@ class ColorSelectionDialog(Gtk.ColorSelectionDialog):
 ColorSelectionDialog = override(ColorSelectionDialog)
 __all__.append('ColorSelectionDialog')
 
+
 class FileChooserDialog(Gtk.FileChooserDialog):
     def __init__(self,
                  title=None,
@@ -525,6 +537,7 @@ class FileChooserDialog(Gtk.FileChooserDialog):
 FileChooserDialog = override(FileChooserDialog)
 __all__.append('FileChooserDialog')
 
+
 class FontSelectionDialog(Gtk.FontSelectionDialog):
     def __init__(self, title=None, **kwds):
         Gtk.FontSelectionDialog.__init__(self, title=title, **kwds)
@@ -532,6 +545,7 @@ class FontSelectionDialog(Gtk.FontSelectionDialog):
 FontSelectionDialog = override(FontSelectionDialog)
 __all__.append('FontSelectionDialog')
 
+
 class RecentChooserDialog(Gtk.RecentChooserDialog):
     def __init__(self,
                  title=None,
@@ -550,6 +564,7 @@ class RecentChooserDialog(Gtk.RecentChooserDialog):
 RecentChooserDialog = override(RecentChooserDialog)
 __all__.append('RecentChooserDialog')
 
+
 class IconView(Gtk.IconView):
 
     def __init__(self, model=None, **kwds):
@@ -593,6 +608,7 @@ class IMContext(Gtk.IMContext):
 IMContext = override(IMContext)
 __all__.append('IMContext')
 
+
 class RecentInfo(Gtk.RecentInfo):
 
     def get_application_info(self, app_name):
@@ -603,6 +619,7 @@ class RecentInfo(Gtk.RecentInfo):
 RecentInfo = override(RecentInfo)
 __all__.append('RecentInfo')
 
+
 class TextBuffer(Gtk.TextBuffer):
     def _get_or_create_tag_table(self):
         table = self.get_tag_table()
@@ -644,7 +661,7 @@ class TextBuffer(Gtk.TextBuffer):
         Gtk.TextBuffer.set_text(self, text, length)
 
     def insert(self, iter, text, length=-1):
-        if not isinstance(text , _basestring):
+        if not isinstance(text, _basestring):
             raise TypeError('text must be a string, not %s' % type(text))
 
         Gtk.TextBuffer.insert(self, iter, text, length)
@@ -676,7 +693,7 @@ class TextBuffer(Gtk.TextBuffer):
         self.insert_with_tags(iter, text, *tag_objs)
 
     def insert_at_cursor(self, text, length=-1):
-        if not isinstance(text , _basestring):
+        if not isinstance(text, _basestring):
             raise TypeError('text must be a string, not %s' % type(text))
 
         Gtk.TextBuffer.insert_at_cursor(self, text, length)
@@ -691,6 +708,7 @@ class TextBuffer(Gtk.TextBuffer):
 TextBuffer = override(TextBuffer)
 __all__.append('TextBuffer')
 
+
 class TextIter(Gtk.TextIter):
 
     def forward_search(self, string, flags, limit):
@@ -721,6 +739,7 @@ class TextIter(Gtk.TextIter):
 TextIter = override(TextIter)
 __all__.append('TextIter')
 
+
 class TreeModel(Gtk.TreeModel):
     def __len__(self):
         return self.iter_n_children(None)
@@ -731,9 +750,9 @@ class TreeModel(Gtk.TreeModel):
     # alias for Python 2.x object protocol
     __nonzero__ = __bool__
 
-    def _getiter(self, key):
+    def __getitem__(self, key):
         if isinstance(key, Gtk.TreeIter):
-            return key
+            return TreeModelRow(self, key)
         elif isinstance(key, int) and key < 0:
             index = len(self) + key
             if index < 0:
@@ -742,26 +761,18 @@ class TreeModel(Gtk.TreeModel):
                 aiter = self.get_iter(index)
             except ValueError:
                 raise IndexError("could not find tree path '%s'" % key)
-            return aiter
+            return TreeModelRow(self, aiter)
         else:
             try:
                 aiter = self.get_iter(key)
             except ValueError:
                 raise IndexError("could not find tree path '%s'" % key)
-            return aiter
-
-    def __getitem__(self, key):
-        aiter = self._getiter(key)
-        return TreeModelRow(self, aiter)
+            return TreeModelRow(self, aiter)
 
     def __setitem__(self, key, value):
         row = self[key]
         self.set_row(row.iter, value)
 
-    def __delitem__(self, key):
-        aiter = self._getiter(key)
-        self.remove(aiter)
-
     def __iter__(self):
         return TreeModelRowIter(self, self.get_iter_first())
 
@@ -837,89 +848,89 @@ class TreeModel(Gtk.TreeModel):
         for column in columns:
             value = row[column]
             if value is None:
-               continue  # None means skip this row
+                continue  # None means skip this row
 
             self.set_value(treeiter, column, value)
 
     def _convert_value(self, column, value):
-            if value is None:
-                return None
+        if value is None:
+            return None
 
-            # we may need to convert to a basic type
-            type_ = self.get_column_type(column)
-            if type_ == GObject.TYPE_STRING:
-                if isinstance(value, str):
-                    value = str(value)
-                elif sys.version_info < (3, 0):
-                    if isinstance(value, unicode):
-                        value = value.encode('UTF-8')
-                    else:
-                        raise ValueError('Expected string or unicode for column %i but got %s%s' % (column, value, type(value)))
+        # we may need to convert to a basic type
+        type_ = self.get_column_type(column)
+        if type_ == GObject.TYPE_STRING:
+            if isinstance(value, str):
+                value = str(value)
+            elif sys.version_info < (3, 0):
+                if isinstance(value, unicode):
+                    value = value.encode('UTF-8')
                 else:
-                    raise ValueError('Expected a string for column %i but got %s' % (column, type(value)))
-            elif type_ == GObject.TYPE_FLOAT or type_ == GObject.TYPE_DOUBLE:
-                if isinstance(value, float):
-                    value = float(value)
+                    raise ValueError('Expected string or unicode for column %i but got %s%s' % (column, value, type(value)))
+            else:
+                raise ValueError('Expected a string for column %i but got %s' % (column, type(value)))
+        elif type_ == GObject.TYPE_FLOAT or type_ == GObject.TYPE_DOUBLE:
+            if isinstance(value, float):
+                value = float(value)
+            else:
+                raise ValueError('Expected a float for column %i but got %s' % (column, type(value)))
+        elif type_ == GObject.TYPE_LONG or type_ == GObject.TYPE_INT:
+            if isinstance(value, int):
+                value = int(value)
+            elif sys.version_info < (3, 0):
+                if isinstance(value, long):
+                    value = long(value)
                 else:
-                    raise ValueError('Expected a float for column %i but got %s' % (column, type(value)))
-            elif type_ == GObject.TYPE_LONG or type_ == GObject.TYPE_INT:
-                if isinstance(value, int):
-                    value = int(value)
-                elif sys.version_info < (3, 0):
-                    if isinstance(value, long):
-                        value = long(value)
-                    else:
-                        raise ValueError('Expected an long for column %i but got %s' % (column, type(value)))
-                else:
-                    raise ValueError('Expected an integer for column %i but got %s' % (column, type(value)))
-            elif type_ == GObject.TYPE_BOOLEAN:
-                cmp_classes = [int]
+                    raise ValueError('Expected an long for column %i but got %s' % (column, type(value)))
+            else:
+                raise ValueError('Expected an integer for column %i but got %s' % (column, type(value)))
+        elif type_ == GObject.TYPE_BOOLEAN:
+            cmp_classes = [int]
+            if sys.version_info < (3, 0):
+                cmp_classes.append(long)
+
+            if isinstance(value, tuple(cmp_classes)):
+                value = bool(value)
+            else:
+                raise ValueError('Expected a bool for column %i but got %s' % (column, type(value)))
+        else:
+            # use GValues directly to marshal to the correct type
+            # standard object checks should take care of validation
+            # so we don't have to do it here
+            value_container = GObject.Value()
+            value_container.init(type_)
+            if type_ == GObject.TYPE_CHAR:
+                value_container.set_char(value)
+                value = value_container
+            elif type_ == GObject.TYPE_UCHAR:
+                value_container.set_uchar(value)
+                value = value_container
+            elif type_ == GObject.TYPE_UNICHAR:
+                cmp_classes = [str]
                 if sys.version_info < (3, 0):
-                    cmp_classes.append(long)
+                    cmp_classes.append(unicode)
 
                 if isinstance(value, tuple(cmp_classes)):
-                    value = bool(value)
-                else:
-                    raise ValueError('Expected a bool for column %i but got %s' % (column, type(value)))
-            else:
-                # use GValues directly to marshal to the correct type
-                # standard object checks should take care of validation
-                # so we don't have to do it here
-                value_container = GObject.Value()
-                value_container.init(type_)
-                if type_ == GObject.TYPE_CHAR:
-                    value_container.set_char(value)
-                    value = value_container
-                elif type_ == GObject.TYPE_UCHAR:
-                    value_container.set_uchar(value)
-                    value = value_container
-                elif type_ == GObject.TYPE_UNICHAR:
-                    cmp_classes = [str]
-                    if sys.version_info < (3, 0):
-                        cmp_classes.append(unicode)
+                    value = ord(value[0])
 
-                    if isinstance(value, tuple(cmp_classes)):
-                        value = ord(value[0])
+                value_container.set_uint(value)
+                value = value_container
+            elif type_ == GObject.TYPE_UINT:
+                value_container.set_uint(value)
+                value = value_container
+            elif type_ == GObject.TYPE_ULONG:
+                value_container.set_ulong(value)
+                value = value_container
+            elif type_ == GObject.TYPE_INT64:
+                value_container.set_int64(value)
+                value = value_container
+            elif type_ == GObject.TYPE_UINT64:
+                value_container.set_uint64(value)
+                value = value_container
+            elif type_ == GObject.TYPE_PYOBJECT:
+                value_container.set_boxed(value)
+                value = value_container
 
-                    value_container.set_uint(value)
-                    value = value_container
-                elif type_ == GObject.TYPE_UINT:
-                    value_container.set_uint(value)
-                    value = value_container
-                elif type_ == GObject.TYPE_ULONG:
-                    value_container.set_ulong(value)
-                    value = value_container
-                elif type_ == GObject.TYPE_INT64:
-                    value_container.set_int64(value)
-                    value = value_container
-                elif type_ == GObject.TYPE_UINT64:
-                    value_container.set_uint64(value)
-                    value = value_container
-                elif type_ == GObject.TYPE_PYOBJECT:
-                    value_container.set_boxed(value)
-                    value = value_container
-
-            return value
+        return value
 
     def get(self, treeiter, *columns):
         n_columns = self.get_n_columns()
@@ -942,6 +953,7 @@ class TreeModel(Gtk.TreeModel):
 TreeModel = override(TreeModel)
 __all__.append('TreeModel')
 
+
 class TreeSortable(Gtk.TreeSortable, ):
 
     def get_sort_column_id(self):
@@ -960,6 +972,7 @@ class TreeSortable(Gtk.TreeSortable, ):
 TreeSortable = override(TreeSortable)
 __all__.append('TreeSortable')
 
+
 class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
     def __init__(self, *column_types):
         Gtk.ListStore.__init__(self)
@@ -990,6 +1003,7 @@ class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
 
     # FIXME: sends two signals; check if this can use an atomic
     # insert_with_valuesv()
+
     def insert_before(self, sibling, row=None):
         treeiter = Gtk.ListStore.insert_before(self, sibling)
 
@@ -1000,6 +1014,7 @@ class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
 
     # FIXME: sends two signals; check if this can use an atomic
     # insert_with_valuesv()
+
     def insert_after(self, sibling, row=None):
         treeiter = Gtk.ListStore.insert_after(self, sibling)
 
@@ -1029,7 +1044,7 @@ class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
                 _set_lists(columns, values)
             elif isinstance(args[0], (tuple, list)):
                 if len(args) != 2:
-                    raise TypeError('Too many arguments');
+                    raise TypeError('Too many arguments')
                 _set_lists(args[0], args[1])
             elif isinstance(args[0], dict):
                 columns = args[0].keys()
@@ -1041,6 +1056,7 @@ class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
 ListStore = override(ListStore)
 __all__.append('ListStore')
 
+
 class TreeModelRow(object):
 
     def __init__(self, model, iter_or_path):
@@ -1108,7 +1124,7 @@ class TreeModelRow(object):
                     "attempt to assign sequence of size %d to slice of size %d"
                         % (len(value), len(indexList)))
 
-            for i,v in enumerate(indexList):
+            for i, v in enumerate(indexList):
                 self.model.set_value(self.iter, v, value[i])
         else:
             raise TypeError("index must be an integer or slice, not %s" % type(key).__name__)
@@ -1125,6 +1141,7 @@ class TreeModelRow(object):
 
 __all__.append('TreeModelRow')
 
+
 class TreeModelRowIter(object):
 
     def __init__(self, model, aiter):
@@ -1146,6 +1163,7 @@ class TreeModelRowIter(object):
 
 __all__.append('TreeModelRowIter')
 
+
 class TreePath(Gtk.TreePath):
 
     def __new__(cls, path=0):
@@ -1188,6 +1206,7 @@ class TreePath(Gtk.TreePath):
 TreePath = override(TreePath)
 __all__.append('TreePath')
 
+
 class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
 
     def __init__(self, *column_types):
@@ -1214,6 +1233,7 @@ class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
 
     # FIXME: sends two signals; check if this can use an atomic
     # insert_with_valuesv()
+
     def insert_before(self, parent, sibling, row=None):
         treeiter = Gtk.TreeStore.insert_before(self, parent, sibling)
 
@@ -1224,6 +1244,7 @@ class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
 
     # FIXME: sends two signals; check if this can use an atomic
     # insert_with_valuesv()
+
     def insert_after(self, parent, sibling, row=None):
         treeiter = Gtk.TreeStore.insert_after(self, parent, sibling)
 
@@ -1253,7 +1274,7 @@ class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
                 _set_lists(columns, values)
             elif isinstance(args[0], (tuple, list)):
                 if len(args) != 2:
-                    raise TypeError('Too many arguments');
+                    raise TypeError('Too many arguments')
                 _set_lists(args[0], args[1])
             elif isinstance(args[0], dict):
                 columns = args[0].keys()
@@ -1265,6 +1286,7 @@ class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
 TreeStore = override(TreeStore)
 __all__.append('TreeStore')
 
+
 class TreeView(Gtk.TreeView, Container):
 
     def __init__(self, model=None):
@@ -1325,6 +1347,7 @@ class TreeView(Gtk.TreeView, Container):
 TreeView = override(TreeView)
 __all__.append('TreeView')
 
+
 class TreeViewColumn(Gtk.TreeViewColumn):
     def __init__(self, title='',
                  cell_renderer=None,
@@ -1347,6 +1370,7 @@ class TreeViewColumn(Gtk.TreeViewColumn):
 TreeViewColumn = override(TreeViewColumn)
 __all__.append('TreeViewColumn')
 
+
 class TreeSelection(Gtk.TreeSelection):
 
     def select_path(self, path):
@@ -1362,6 +1386,7 @@ class TreeSelection(Gtk.TreeSelection):
             return (model, None)
 
     # for compatibility with PyGtk
+
     def get_selected_rows(self):
         rows, model = super(TreeSelection, self).get_selected_rows()
         return (model, rows)
@@ -1370,6 +1395,7 @@ class TreeSelection(Gtk.TreeSelection):
 TreeSelection = override(TreeSelection)
 __all__.append('TreeSelection')
 
+
 class Button(Gtk.Button, Container):
     def __init__(self, label=None, stock=None, use_stock=False, use_underline=False, **kwds):
         if stock:
@@ -1381,6 +1407,7 @@ class Button(Gtk.Button, Container):
 Button = override(Button)
 __all__.append('Button')
 
+
 class LinkButton(Gtk.LinkButton):
     def __init__(self, uri, label=None, **kwds):
         Gtk.LinkButton.__init__(self, uri=uri, label=label, **kwds)
@@ -1388,6 +1415,7 @@ class LinkButton(Gtk.LinkButton):
 LinkButton = override(LinkButton)
 __all__.append('LinkButton')
 
+
 class Label(Gtk.Label):
     def __init__(self, label=None, **kwds):
         Gtk.Label.__init__(self, label=label, **kwds)
@@ -1395,6 +1423,7 @@ class Label(Gtk.Label):
 Label = override(Label)
 __all__.append('Label')
 
+
 class Adjustment(Gtk.Adjustment):
     def __init__(self, *args, **kwds):
         arg_names = ('value', 'lower', 'upper',
@@ -1418,6 +1447,7 @@ class Adjustment(Gtk.Adjustment):
 Adjustment = override(Adjustment)
 __all__.append('Adjustment')
 
+
 class Table(Gtk.Table, Container):
     def __init__(self, rows=1, columns=1, homogeneous=False, **kwds):
         if 'n_rows' in kwds:
@@ -1425,15 +1455,16 @@ class Table(Gtk.Table, Container):
 
         if 'n_columns' in kwds:
             columns = kwds.pop('n_columns')
-            
+
         Gtk.Table.__init__(self, n_rows=rows, n_columns=columns, homogeneous=homogeneous, **kwds)
 
-    def attach(self, child, left_attach, right_attach, top_attach, bottom_attach, xoptions=Gtk.AttachOptions.EXPAND|Gtk.AttachOptions.FILL, yoptions=Gtk.AttachOptions.EXPAND|Gtk.AttachOptions.FILL, xpadding=0, ypadding=0):
+    def attach(self, child, left_attach, right_attach, top_attach, bottom_attach, xoptions=Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL, yoptions=Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL, xpadding=0, ypadding=0):
         Gtk.Table.attach(self, child, left_attach, right_attach, top_attach, bottom_attach, xoptions, yoptions, xpadding, ypadding)
 
 Table = override(Table)
 __all__.append('Table')
 
+
 class ScrolledWindow(Gtk.ScrolledWindow):
     def __init__(self, hadjustment=None, vadjustment=None, **kwds):
         Gtk.ScrolledWindow.__init__(self, hadjustment=hadjustment, vadjustment=vadjustment, **kwds)
@@ -1449,6 +1480,7 @@ class HScrollbar(Gtk.HScrollbar):
 HScrollbar = override(HScrollbar)
 __all__.append('HScrollbar')
 
+
 class VScrollbar(Gtk.VScrollbar):
     def __init__(self, adjustment=None, **kwds):
         Gtk.VScrollbar.__init__(self, adjustment=adjustment, **kwds)
@@ -1456,6 +1488,7 @@ class VScrollbar(Gtk.VScrollbar):
 VScrollbar = override(VScrollbar)
 __all__.append('VScrollbar')
 
+
 class Paned(Gtk.Paned):
     def pack1(self, child, resize=False, shrink=True):
         super(Paned, self).pack1(child, resize, shrink)
@@ -1514,11 +1547,15 @@ if Gtk._version != '2.0':
     __all__.append('Menu')
 
 _Gtk_main_quit = Gtk.main_quit
+
+
 @override(Gtk.main_quit)
 def main_quit(*args):
     _Gtk_main_quit()
 
 _Gtk_stock_lookup = Gtk.stock_lookup
+
+
 @override(Gtk.stock_lookup)
 def stock_lookup(*args):
     success, item = _Gtk_stock_lookup(*args)
