@@ -5,17 +5,18 @@ import struct
 import unittest
 
 from gi.repository import GObject
-from  gi.repository.GObject import GType, GEnum, new, PARAM_READWRITE, \
+from  gi.repository.GObject import GType, new, PARAM_READWRITE, \
      PARAM_CONSTRUCT, PARAM_READABLE, PARAM_WRITABLE, PARAM_CONSTRUCT_ONLY
 from gi.repository.GObject import \
-     TYPE_INT, TYPE_UINT, TYPE_LONG, \
-     TYPE_ULONG, TYPE_INT64, TYPE_UINT64
+     TYPE_INT, TYPE_UINT, TYPE_LONG, TYPE_ULONG, TYPE_INT64, \
+     TYPE_UINT64, TYPE_GTYPE, TYPE_INVALID, TYPE_NONE
 from gi.repository.GObject import \
      G_MININT, G_MAXINT, G_MAXUINT, G_MINLONG, G_MAXLONG, \
      G_MAXULONG
 
 from gi.repository import Gio
 from gi.repository import GLib
+from gi.repository import GIMarshallingTests
 
 if sys.version_info < (3, 0):
     TEST_UTF8 = "\xe2\x99\xa5"
@@ -26,22 +27,31 @@ else:
 
 from compathelper import _long
 
+
 class PropertyObject(GObject.GObject):
     normal = GObject.Property(type=str)
     construct = GObject.Property(
         type=str,
-        flags=PARAM_READWRITE|PARAM_CONSTRUCT, default='default')
+        flags=PARAM_READWRITE | PARAM_CONSTRUCT, default='default')
     construct_only = GObject.Property(
         type=str,
-        flags=PARAM_READWRITE|PARAM_CONSTRUCT_ONLY)
+        flags=PARAM_READWRITE | PARAM_CONSTRUCT_ONLY)
     uint64 = GObject.Property(
-        type=TYPE_UINT64, flags=PARAM_READWRITE|PARAM_CONSTRUCT)
+        type=TYPE_UINT64, flags=PARAM_READWRITE | PARAM_CONSTRUCT)
 
     enum = GObject.Property(
         type=Gio.SocketType, default=Gio.SocketType.STREAM)
 
     boxed = GObject.Property(
-        type=GLib.Regex, flags=PARAM_READWRITE|PARAM_CONSTRUCT)
+        type=GLib.Regex, flags=PARAM_READWRITE | PARAM_CONSTRUCT)
+
+    flags = GObject.Property(
+        type=GIMarshallingTests.Flags, flags=PARAM_READWRITE | PARAM_CONSTRUCT,
+        default=GIMarshallingTests.Flags.VALUE1)
+
+    gtype = GObject.Property(
+        type=TYPE_GTYPE, flags=PARAM_READWRITE | PARAM_CONSTRUCT)
+
 
 class TestProperties(unittest.TestCase):
     def testGetSet(self):
@@ -51,10 +61,10 @@ class TestProperties(unittest.TestCase):
 
     def testListWithInstance(self):
         obj = PropertyObject()
-        self.failUnless(hasattr(obj.props, "normal"))
+        self.assertTrue(hasattr(obj.props, "normal"))
 
     def testListWithoutInstance(self):
-        self.failUnless(hasattr(PropertyObject.props, "normal"))
+        self.assertTrue(hasattr(PropertyObject.props, "normal"))
 
     def testSetNoInstance(self):
         def set(obj):
@@ -67,13 +77,15 @@ class TestProperties(unittest.TestCase):
             for pspec in obj:
                 gtype = GType(pspec)
                 self.assertEqual(gtype.parent.name, 'GParam')
-                self.failUnless(pspec.name in ['normal',
+                self.assertTrue(pspec.name in ['normal',
                                                'construct',
                                                'construct-only',
                                                'uint64',
                                                'enum',
+                                               'flags',
+                                               'gtype',
                                                'boxed'])
-            self.assertEqual(len(obj), 6)
+            self.assertEqual(len(obj), 8)
 
     def testNormal(self):
         obj = new(PropertyObject, normal="123")
@@ -131,7 +143,7 @@ class TestProperties(unittest.TestCase):
             class TimeControl(GObject.GObject):
                 __gproperties__ = {
                     'time': (TYPE_UINT64, 'Time', 'Time',
-                             _long(0), (1<<64) - 1, _long(0),
+                             _long(0), (1 << 64) - 1, _long(0),
                              PARAM_READABLE)
                     }
         except OverflowError:
@@ -166,6 +178,58 @@ class TestProperties(unittest.TestCase):
         self.assertRaises(TypeError, GObject.Property, type=Gio.SocketType,
                           default=1)
 
+    def testFlags(self):
+        obj = new(PropertyObject)
+        self.assertEqual(obj.props.flags, GIMarshallingTests.Flags.VALUE1)
+        self.assertEqual(obj.flags, GIMarshallingTests.Flags.VALUE1)
+
+        obj.flags = GIMarshallingTests.Flags.VALUE2 | GIMarshallingTests.Flags.VALUE3
+        self.assertEqual(obj.props.flags, GIMarshallingTests.Flags.VALUE2 | GIMarshallingTests.Flags.VALUE3)
+        self.assertEqual(obj.flags, GIMarshallingTests.Flags.VALUE2 | GIMarshallingTests.Flags.VALUE3)
+
+        self.assertRaises(TypeError, setattr, obj, 'flags', 'foo')
+        self.assertRaises(TypeError, setattr, obj, 'flags', object())
+        self.assertRaises(TypeError, setattr, obj, 'flags', None)
+
+        self.assertRaises(TypeError, GObject.Property,
+                type=GIMarshallingTests.Flags, default='foo')
+        self.assertRaises(TypeError, GObject.Property,
+                type=GIMarshallingTests.Flags, default=object())
+        self.assertRaises(TypeError, GObject.Property,
+                type=GIMarshallingTests.Flags, default=None)
+
+    def testGType(self):
+        obj = new(PropertyObject)
+
+        self.assertEqual(obj.props.gtype, TYPE_NONE)
+        self.assertEqual(obj.gtype, TYPE_NONE)
+
+        obj.gtype = TYPE_UINT64
+        self.assertEqual(obj.props.gtype, TYPE_UINT64)
+        self.assertEqual(obj.gtype, TYPE_UINT64)
+
+        obj.gtype = TYPE_INVALID
+        self.assertEqual(obj.props.gtype, TYPE_INVALID)
+        self.assertEqual(obj.gtype, TYPE_INVALID)
+
+        # GType parameters do not support defaults in GLib
+        self.assertRaises(TypeError, GObject.Property, type=TYPE_GTYPE,
+                default=TYPE_INT)
+
+        # incompatible type
+        self.assertRaises(TypeError, setattr, obj, 'gtype', 'foo')
+        self.assertRaises(TypeError, setattr, obj, 'gtype', object())
+
+        self.assertRaises(TypeError, GObject.Property, type=TYPE_GTYPE,
+                default='foo')
+        self.assertRaises(TypeError, GObject.Property, type=TYPE_GTYPE,
+                default=object())
+
+        # set in constructor
+        obj = new(PropertyObject, gtype=TYPE_UINT)
+        self.assertEqual(obj.props.gtype, TYPE_UINT)
+        self.assertEqual(obj.gtype, TYPE_UINT)
+
     def textBoxed(self):
         obj = new(PropertyObject)
 
@@ -181,6 +245,7 @@ class TestProperties(unittest.TestCase):
         # kiwi code
         def max(c):
             return 2 ** ((8 * struct.calcsize(c)) - 1) - 1
+
         def umax(c):
             return 2 ** (8 * struct.calcsize(c)) - 1
 
@@ -245,7 +310,6 @@ class TestProperties(unittest.TestCase):
             obj.set_property(key, max)
             self.assertEqual(obj.get_property(key), max)
 
-
     def testMulti(self):
         obj = PropertyObject()
         obj.set_properties(normal="foo",
@@ -254,6 +318,7 @@ class TestProperties(unittest.TestCase):
         self.assertEqual(normal, "foo")
         self.assertEqual(uint64, 7)
 
+
 class TestProperty(unittest.TestCase):
     def testSimple(self):
         class C(GObject.GObject):
@@ -262,10 +327,10 @@ class TestProperty(unittest.TestCase):
             float = GObject.Property(type=float)
             long = GObject.Property(type=_long)
 
-        self.failUnless(hasattr(C.props, 'str'))
-        self.failUnless(hasattr(C.props, 'int'))
-        self.failUnless(hasattr(C.props, 'float'))
-        self.failUnless(hasattr(C.props, 'long'))
+        self.assertTrue(hasattr(C.props, 'str'))
+        self.assertTrue(hasattr(C.props, 'int'))
+        self.assertTrue(hasattr(C.props, 'float'))
+        self.assertTrue(hasattr(C.props, 'long'))
 
         o = C()
         self.assertEqual(o.str, '')
@@ -305,19 +370,21 @@ class TestProperty(unittest.TestCase):
                 GObject.GObject.__init__(self)
 
         o = C()
-        self.assertEquals(o._value, None)
+        self.assertEqual(o._value, None)
         o.prop = 'bar'
-        self.assertEquals(o._value, 'bar')
+        self.assertEqual(o._value, 'bar')
         self.assertRaises(TypeError, getattr, o, 'prop')
 
     def testDecoratorDefault(self):
         class C(GObject.GObject):
             _value = 'value'
+
             @GObject.Property
             def value(self):
                 return self._value
+
             @value.setter
-            def value(self, value):
+            def value_setter(self, value):
                 self._value = value
 
         o = C()
@@ -329,11 +396,13 @@ class TestProperty(unittest.TestCase):
     def testDecoratorWithCall(self):
         class C(GObject.GObject):
             _value = 1
+
             @GObject.Property(type=int, default=1, minimum=1, maximum=10)
             def typedValue(self):
                 return self._value
+
             @typedValue.setter
-            def typedValue(self, value):
+            def typedValue_setter(self, value):
                 self._value = value
 
         o = C()
@@ -357,8 +426,8 @@ class TestProperty(unittest.TestCase):
         self.assertRaises(TypeError, GObject.Property, flags=-10)
 
     def testDefaults(self):
-        p1 = GObject.Property(type=bool, default=True)
-        p2 = GObject.Property(type=bool, default=False)
+        GObject.Property(type=bool, default=True)
+        GObject.Property(type=bool, default=False)
 
     def testNameWithUnderscore(self):
         class C(GObject.GObject):
@@ -391,13 +460,13 @@ class TestProperty(unittest.TestCase):
 
             # Lower than minimum
             self.assertRaises(TypeError,
-                              GObject.Property, type=gtype, minimum=min-1,
+                              GObject.Property, type=gtype, minimum=min - 1,
                               maximum=max)
 
             # Higher than maximum
             self.assertRaises(TypeError,
                               GObject.Property, type=gtype, minimum=min,
-                              maximum=max+1)
+                              maximum=max + 1)
 
     def testMinMax(self):
         class C(GObject.GObject):
@@ -407,28 +476,34 @@ class TestProperty(unittest.TestCase):
             def __init__(self):
                 GObject.GObject.__init__(self)
 
-        o = C()
-        self.assertEqual(o.prop_int, 1)
+        # we test known-bad values here which cause Gtk-WARNING logs.
+        # Explicitly allow these for this test.
+        old_mask = GLib.log_set_always_fatal(GLib.LogLevelFlags.LEVEL_CRITICAL)
+        try:
+            o = C()
+            self.assertEqual(o.prop_int, 1)
 
-        o.prop_int = 5
-        self.assertEqual(o.prop_int, 5)
+            o.prop_int = 5
+            self.assertEqual(o.prop_int, 5)
 
-        o.prop_int = 0
-        self.assertEqual(o.prop_int, 5)
+            o.prop_int = 0
+            self.assertEqual(o.prop_int, 5)
 
-        o.prop_int = 101
-        self.assertEqual(o.prop_int, 5)
+            o.prop_int = 101
+            self.assertEqual(o.prop_int, 5)
 
-        self.assertEqual(o.prop_float, 1.1)
+            self.assertEqual(o.prop_float, 1.1)
 
-        o.prop_float = 7.75
-        self.assertEqual(o.prop_float, 7.75)
+            o.prop_float = 7.75
+            self.assertEqual(o.prop_float, 7.75)
 
-        o.prop_float = 0.09
-        self.assertEqual(o.prop_float, 7.75)
+            o.prop_float = 0.09
+            self.assertEqual(o.prop_float, 7.75)
 
-        o.prop_float = 10.51
-        self.assertEqual(o.prop_float, 7.75)
+            o.prop_float = 10.51
+            self.assertEqual(o.prop_float, 7.75)
+        finally:
+            GLib.log_set_always_fatal(old_mask)
 
     def testMultipleInstances(self):
         class C(GObject.GObject):
@@ -462,7 +537,7 @@ class TestProperty(unittest.TestCase):
         class PropertyObjectSubclass(GObject.GObject):
             obj = GObject.Property(type=ObjectSubclass)
 
-        obj1 = PropertyObjectSubclass(obj=ObjectSubclass())
+        PropertyObjectSubclass(obj=ObjectSubclass())
 
     def testPropertySubclass(self):
         # test for #470718
@@ -474,9 +549,9 @@ class TestProperty(unittest.TestCase):
 
         b = B()
         b.prop2 = 10
-        self.assertEquals(b.prop2, 10)
+        self.assertEqual(b.prop2, 10)
         b.prop1 = 20
-        self.assertEquals(b.prop1, 20)
+        self.assertEqual(b.prop1, 20)
 
     def testPropertySubclassCustomSetter(self):
         # test for #523352
@@ -491,13 +566,13 @@ class TestProperty(unittest.TestCase):
             second = GObject.Property(type=str, getter=get_second)
 
         a = A()
-        self.assertEquals(a.first, 'first')
+        self.assertEqual(a.first, 'first')
         self.assertRaises(TypeError, setattr, a, 'first', 'foo')
 
         b = B()
-        self.assertEquals(b.first, 'first')
+        self.assertEqual(b.first, 'first')
         self.assertRaises(TypeError, setattr, b, 'first', 'foo')
-        self.assertEquals(b.second, 'second')
+        self.assertEqual(b.second, 'second')
         self.assertRaises(TypeError, setattr, b, 'second', 'foo')
 
     def testPropertySubclassCustomSetterError(self):
@@ -515,12 +590,14 @@ class TestProperty(unittest.TestCase):
             raise AssertionError
 
     # Bug 587637.
+
     def test_float_min(self):
         GObject.Property(type=float, minimum=-1)
         GObject.Property(type=GObject.TYPE_FLOAT, minimum=-1)
         GObject.Property(type=GObject.TYPE_DOUBLE, minimum=-1)
 
     # Bug 644039
+
     def testReferenceCount(self):
         # We can check directly if an object gets finalized, so we will
         # observe it indirectly through the refcount of a member object.
@@ -533,14 +610,14 @@ class TestProperty(unittest.TestCase):
         # want to observe. Its refcount is increased by one.
         t = PropertyObject(normal="test")
         t.o = o
-        self.assertEquals(sys.getrefcount(o), rc + 1)
+        self.assertEqual(sys.getrefcount(o), rc + 1)
 
         # Now we want to ensure we do not leak any references to our
         # object with properties. If no ref is leaked, then when deleting
         # the local reference to this object, its reference count shoud
         # drop to zero, and our dummy object should loose one reference.
         del t
-        self.assertEquals(sys.getrefcount(o), rc)
+        self.assertEqual(sys.getrefcount(o), rc)
 
     def testDocStringAsBlurb(self):
         class C(GObject.GObject):
